/++
 + Firebird接続ライブラリ Version: 0.6.1
 +
 + Blob実装モジュール
 +
 + Copyright(C) 2009-2011 Takuya Kurosawa
 +/
module jp.ku6.firebird.blob;

import jp.ku6.firebird.core;

/++
 + blobデータの実体をメモリ上に持つクラス
 +
 + blobデータをすべてメモリ上に持つため、メモリ使用量に注意すること
 +/
class MemoryBlob : Blob
{
	/++
	 + コンストラクタ
	 +/
	this()
	{
		data_ = new ubyte[8096];
		segmentSize_ = 8096;
	}

	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  data = blobデータの実体
	 +  segmentSize = セグメントのサイズ
	 +/
	this(ubyte[] data, ushort segmentSize)
	{
		data_ = data;
		segmentSize_ = segmentSize;
		length_ = cast(uint)data_.length;
	}

	/++
	 + blobデータの実体を取得
	 +
	 + Returns:
	 +  blobデータの実体
	 +/
	ubyte[] data()
	{
		return data_[0 .. length_];
	}

	override
	void resetReadSegment()
	{
		readSegmentStart_ = 0;
	}

	override
	void resetWriteSegment()
	{
		writeSegmentStart_ = 0;
	}

	override
	const(ubyte)[] getReadSegment()
	{
		uint start = readSegmentStart_;
		if (start >= data_.length)
			return null;

		uint size = segmentSize_;
		if (start + size >= data_.length)
			size = cast(uint)data_.length - start;

		readSegmentStart_ += size;

		return data_[start .. start + size];
	}

	override
	ubyte[] getWriteSegment()
	{
		uint start = writeSegmentStart_;
		if (start >= data_.length)
		{
			data_.length = data_.length + segmentSize_;
			return data_[start .. $];
		}

		uint size = segmentSize_;
		if (start + size >= data_.length)
			size = cast(uint)data_.length - start;

		return data_[start .. start + size];
	}

	override
	void appriseWrittenSize(ushort size)
	{
		writeSegmentStart_ += size;
		length_ = writeSegmentStart_;
	}

private:

	// セグメントのサイズ
	ushort segmentSize_;

	// 読み出しセグメントの開始位置
	uint readSegmentStart_;

	// 書き込みセグメントの開始位置
	uint writeSegmentStart_;

	// blobデータの実体
	ubyte[] data_;

	// blobデータの実体のサイズ 必ずしもdata_.lengthと一致しない
	uint length_;
}
